import java.awt.geom.Point2D;
import java.io.IOException;

import com.change_vision.jude.api.inf.editor.BasicModelEditor;
import com.change_vision.jude.api.inf.editor.ClassDiagramEditor;
import com.change_vision.jude.api.inf.editor.ModelEditorFactory;
import com.change_vision.jude.api.inf.editor.TransactionManager;
import com.change_vision.jude.api.inf.exception.InvalidEditingException;
import com.change_vision.jude.api.inf.exception.LicenseNotFoundException;
import com.change_vision.jude.api.inf.exception.ProjectLockedException;
import com.change_vision.jude.api.inf.exception.ProjectNotFoundException;
import com.change_vision.jude.api.inf.model.IAssociation;
import com.change_vision.jude.api.inf.model.IClass;
import com.change_vision.jude.api.inf.model.IGeneralization;
import com.change_vision.jude.api.inf.model.IModel;
import com.change_vision.jude.api.inf.model.IPackage;
import com.change_vision.jude.api.inf.model.IRealization;
import com.change_vision.jude.api.inf.presentation.INodePresentation;
import com.change_vision.jude.api.inf.project.ProjectAccessor;
import com.change_vision.jude.api.inf.project.ProjectAccessorFactory;


/**
 * APIŃNX}AсAe}vf쐬AAstah̃vWFNgt@C쐬TvR[hB 
 */
public class APIForCreatingPresentationsSample {
	
	//installer
	private static final String PROJECT_PATH = "./SampleModel.asta";
	//eclipse
	//private static final String PROJECT_PATH = "api_sample\\simpleDgmEdit\\SampleModel.asta";
	
	public static void main(String[] args) {
		
		try {
			System.out.println("Creating new project...");
			
            // vWFNg쐬āAN_ƂȂ郂f擾
			ProjectAccessor prjAccessor = ProjectAccessorFactory.getProjectAccessor();
			prjAccessor.create(PROJECT_PATH);
			IModel project = prjAccessor.getProject();
			
			System.out.println("Creating new elements in the project...");
			
			// gUNV̊Jn
			TransactionManager.beginTransaction();
			
			// -----<<< f̍쐬 >>>-----
			BasicModelEditor basicModelEditor = ModelEditorFactory.getBasicModelEditor();
			
			// pbP[W쐬
			IPackage packageA = basicModelEditor.createPackage(project, "PackageA");
			
			// w̃pbP[WɃNX쐬
			IClass classA = basicModelEditor.createClass(packageA, "ClassA");
			// NXɑǉ
			basicModelEditor.createAttribute(classA, "attribute0", "int");
			// NXɑǉ
			basicModelEditor.createOperation(classA, "operation0", "void");
			
			// w̃pbP[WɃNXC^[tF[X쐬
			IClass classB = basicModelEditor.createClass(packageA, "ClassB");
			IClass classC = basicModelEditor.createClass(packageA, "ClassC");
			IClass interfaceA = basicModelEditor.createInterface(packageA, "InterfaceA");
			
			// NXԂ̊֘A쐬
			IAssociation association 
				= basicModelEditor.createAssociation(classA, classB, "association name",
					"classA end", "classB end");
			// NXԂ̔ĉ쐬
			IGeneralization generalization 
				= basicModelEditor.createGeneralization(classC, classA, "generalization name");
			// NXԂ̎쐬
			IRealization realization 
				= basicModelEditor.createRealization(classB, interfaceA, "realization name");
			
			// -----<<< }̍쐬 >>>-----
			ClassDiagramEditor diagramEditor 
				= ProjectAccessorFactory.getProjectAccessor().getDiagramEditorFactory().getClassDiagramEditor();

			// w̃pbP[WɃNX}쐬
			diagramEditor.createClassDiagram(packageA, "Class DiagramA");

			// NX}ɃNXEC^[tF[Xv[e[Vǉ
			Point2D locationA = new Point2D.Double(10.0d, 10.0d);
			INodePresentation classAPs = diagramEditor.createNodePresentation(classA, locationA);
			
			Point2D locationB = new Point2D.Double(300.0d, 25.0d);
			INodePresentation classBPs = diagramEditor.createNodePresentation(classB, locationB);
			
			Point2D locationC = new Point2D.Double(45.0d, 200.0d);
			INodePresentation classCPs = diagramEditor.createNodePresentation(classC, locationC);
			
			Point2D locationD = new Point2D.Double(500.0d, 29.0d);
			INodePresentation interfaceAPs = diagramEditor.createNodePresentation(interfaceA, locationD);
						
			// NX}Ɋ֘Av[e[Vǉ
			diagramEditor.createLinkPresentation(association, classAPs, classBPs);
			// NX}ɔĉv[e[Vǉ
			diagramEditor.createLinkPresentation(generalization, classAPs, classCPs);
			// NX}Ɏv[e[Vǉ
			diagramEditor.createLinkPresentation(realization, interfaceAPs, classBPs);
			
			// NXv[e[VɐFǉ
			classAPs.setProperty("fill.color", "#FF0000");
			
			// gUNV̏I
			TransactionManager.endTransaction();
			
			// vWFNgۑ
			prjAccessor.save();
			
			// vWFNg
			prjAccessor.close();
			
			System.out.println("Finished");
			
		} catch (LicenseNotFoundException e) {
			e.printStackTrace();
		} catch (ProjectNotFoundException e) {
			e.printStackTrace();
		} catch (ProjectLockedException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidEditingException e) {
			// gUNV̕
			TransactionManager.abortTransaction();
			// sҏW̗ObZ[W擾
			System.err.print(e.getMessage());
			e.printStackTrace();
		} catch (IOException e ) {
			e.printStackTrace();
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
}